\begin{code}
module Genus2MCG where
import Genus2
import Genus2Moves
\end{code}

Standard generators for the mapping class group: Dehn twists around
particular curves.  The standard form for the pair of pants
decomposition is a barbell; each of the standard generators goes from
a barbell pants decomposition to itself.

\begin{code}
stdGens :: [[Move]]
stdGens = [[(1,2),(1,0),(2,1),(2,0)],
	   [(1,0),(1,2),(2,1),(2,0)],
	   [(2,0),(2,2)],
	   [(3,0),(3,2),(2,1),(2,0)],
	   [(3,2),(3,0),(2,1),(2,0)]]

invGens :: [[Move]]
invGens = [[(1,-2),(1,0),(2,1),(2,0)],
	   [(1,0),(1,-2),(2,1),(2,0)],
	   [(2,0),(2,-2)],
	   [(3,0),(3,-2),(2,1),(2,0)],
	   [(3,-2),(3,0),(2,1),(2,0)]]
\end{code}

A word in the mapping class group is represent by a sequence of
integers; 1..5 for the standard generators, and -1..-5 for their
inverses.

\begin{code}
type MCGen = Int
type MCWord = [MCGen]

stdGen :: MCGen -> [Move]
stdGen x | x > 0 && x <= 5 = stdGens!!(x-1)
stdGen x | x < 0 && x >= -5 = invGens!!(-1-x)

doMCG :: MCWord -> Genus2 -> Genus2
doMCG is = doMoves $ concat $ map (stdGen) is
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
