\begin{code}
module Genus2Moves where
import DT
import PIL
import Genus2
import BasicMoves
\end{code}

A move is a pair (edge to transform, amount to twist).  The amount to
twist is relative to a natural basis, and is measured in _eighth_
twists; it should always be odd, so that it can rounded unambiguously
to the nearest half twist or full twist, depending on the type of
move.

\begin{code}
type Move = (Int, Integer)

moveT :: Move -> Genus2T -> Genus2T
moveT (1,t) (g2@Barbell {},(n1,n2,n3)) =
    let t' = roundTwistF (t+n1)
	g2' = move (1,t') g2
    in (g2',(0,2+n2,n3))
moveT (3,t) g2t@(Barbell {},_) = reflectT $ moveT (1,t) $ reflectT g2t
moveT (2,t) (g2@Barbell {},(n1,n2,n3)) =
    let t' = roundTwistH (t+n2)
	g2' = move (2,t') g2
    in (g2', (n1,0,n3))
moveT (2,t) (g2@Theta {},(n1,n2,n3)) =
    let t' = roundTwistH (t+n2)
	g2' = move (2,t') g2
    in if even t' then (g2',(n1,0,n3))
                  else (g2',(n1,4,n3))
moveT (1,t) g2t@(Theta {},_) = moveT (2,t) $ rotateT g2t
moveT (3,t) g2t@(Theta {},_) = moveT (2,t) $ rotateT $ rotateT g2t

move :: Move -> Genus2 -> Genus2
move (1,t) (Barbell e1 e2 e3) | even t =
    let (e1', de2) = firstTrans (m e2) (dehnTwist t e1)
    in Barbell e1' (twist de2 e2) e3
move (1,t) g2@(Barbell _ _ _) | odd t = move (1,t+1) g2
move (3,t) g2@(Barbell _ _ _) = reflect $ move (1,t) $ reflect $ g2
move (2,t) (Barbell e1 e2 e3) =
    let (e2', (de1,de1',de3,de3')) =
			secondTrans (m e1,m e1,m e3,m e3) (dehnTwist t e2)
    in Theta (twist (de1+de1') e1) e2' (twist (de3+de3') e3)
move (2,t) (Theta e1 e2 e3) | even t =
    let (e2', (de1, de3, de3', de1')) =
			secondTrans (m e1,m e3,m e3,m e1) (dehnTwist t e2)
	in Barbell (twist (de1+de1') e1) e2' (twist (de3+de3') e3)
move (2,t) (Theta e1 e2 e3) | odd t =
    let (e2', (de1,de3,de1',de3')) =
			secondTrans (m e1,m e3,m e1,m e3) (dehnTwist t e2)
    in Theta (twist (de1+de1') e1) (dehnTwist (-1) e2') (twist (de3+de3') e3)
move (1,t) (Theta e1 e2 e3) = move (2,t) (Theta e3 e1 e2)
move (3,t) (Theta e1 e2 e3) = move (2,t) (Theta e2 e3 e1)
\end{code}

Round to the nearest full or half twist.

\begin{code}	      
roundTwistF :: Integer -> Integer
roundTwistF n | odd n = ((n+4) `div` 8) * 2
	      | otherwise = error "roundTwistF: parity error"

roundTwistH :: Integer -> Integer
roundTwistH n | odd n = (n+2) `div` 4
	      | otherwise = error "roundTwistF: parity error"
\end{code}

Some very simple curves to get started with.

\begin{code}
emptyAnnulus, simpleAnnulus :: DTAnnulus
emptyAnnulus = DTAnnulus {m = 0, t = 0}
simpleAnnulus = DTAnnulus {m = 0, t = 2}
simpleCurve :: Genus2
simpleCurve = Barbell emptyAnnulus emptyAnnulus (DTAnnulus 1 0)
simpleCurveT :: Genus2T
simpleCurveT = (simpleCurve,(0,0,0))
\end{code}

Perform a sequence of moves.

\begin{code}
doMovesT :: [Move] -> Genus2T -> Genus2T
doMovesT ms g = foldl (flip moveT) g ms
doMoves :: [Move] -> Genus2 -> Genus2
doMoves ms g = foldl (flip move) g ms
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
