\begin{code}
module Monoid where

import Util(reduceRepeat)

infixl 7 &
\end{code}

A _monoid_ is a set with an associative operation (&) and a unit
'unit'.

\begin{code}
class Monoid a where
  unit :: a
  (&) :: a -> a -> a
  pow :: a -> Integer -> a
  pow x n = reduceRepeat (&) unit x n

\end{code}

\begin{code}
prop_lunit a = unit & a == a
prop_runit a = a & unit == a
prop_assoc a b c = (a & b) & c == a & (b & c)
\end{code}

A _reversible monoid_ additionally has a monoid anti-involution 'rev':

\begin{code}
class (Monoid a) => RevMonoid a where
  rev :: a -> a
\end{code}

\begin{code}
prop_revRev a = rev (rev a) == a
prop_revProd a b = rev (a & b) == rev b & rev a
\end{code}

Properties of a monoid morphism:

\begin{code}
prop_morph_unit f x = f unit == unit
prop_morph_prod f x y = f x & f y == f (x & y)
prop_morph_rev f x = rev (f x) == f (rev x)
\end{code}

Simple Instances of monoids
---------------------------

Trivial:

\begin{code}
instance Monoid () where
    unit = ()
    a & b = ()

instance RevMonoid () where
    rev a = ()
\end{code}

Integers:

\begin{code}
instance Monoid Integer where
  unit = 0
  (&) = (+)
  pow = (*)

instance RevMonoid Integer where
  rev = negate
\end{code}

Coproduct monoid:

\begin{code}
instance (Monoid a, Monoid b) => Monoid (a, b) where
  unit = (unit, unit)
  (a1, b1) & (a2, b2) = (a1 & a2, b1 & b2)
  (a1, b1) `pow` n = (a1 `pow` n, b1 `pow` n)

instance (RevMonoid a, RevMonoid b) => RevMonoid (a, b) where
  rev (a, b) = (rev a, rev b)
\end{code}

Copyright 2002, 2004, 2005 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
