\begin{code}
module RandomMoves where
import Genus2
import Genus2Moves
import Genus2MCG
import Random
import DT
import Util(group)
\end{code}

Generate normal variables with given variance.  We could take more
care with the precision, adding more bits as needed.

\begin{code}
normals :: RandomGen g => Double -> g -> [Double]
normals var g =
  let pts :: [(Double,Double)]
      pts = map (\[x,y]->(x,y)) $ group 2 $ map (\x -> 2*x-1) $ randoms g
      f :: (Double, Double) -> [Double]
      f (x,y) = let s = x*x+y*y
		    t = var*sqrt (-2 * log s/s)
		in if s >= 1 || s == 0 then [] else [x*t,y*t]
  in concat $ map f pts
\end{code}

Make random moves, with twists either normally distributed or
uniformly distributed in some interval.

\begin{code}
randMoves :: RandomGen g => Double -> Int -> g -> [[Move]]
randMoves var len g =
    let (g', g'') = split g
	moves = randomRs (1,3) g'
	twists = map ((\x->2*x+1).floor) (normals (4*var) g'')
    in group len $ zip moves twists

randUnifMoves :: RandomGen g => Integer -> Int -> g -> [[Move]]
randUnifMoves var len g =
    let (g', g'') = split g
	moves = randomRs (1,3) g'
	twists = map (\x->2*x+1) $ randomRs (-var,var-1) g''
    in group len $ zip moves twists
\end{code}

We might also want to generate random words in the mapping class group.

\begin{code}
randWords :: RandomGen g => Int -> g -> [MCWord]
randWords len g =
    let f x | x <= 0    = x-1
	    | otherwise = x
	l = randomRs (-4,5) g
    in group len $ map f l
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
