\begin{code}
module Util where
import Random
import Bits
\end{code}

'reduceRepeat op a0 a n' is very similar to 'foldl op (a0:repeat n a)'
(when 'op' is associative), but much more efficient, since it uses the
standard algorithm for powers.

\begin{code}
reduceRepeat :: (a -> a -> a) -> a -> a -> Integer -> a
reduceRepeat op a0 a 0 = a0
reduceRepeat op a0 a n
    | even n = reduceRepeat op a0          (a `op` a) (n `div` 2)
    | odd n  = reduceRepeat op (a0 `op` a) (a `op` a) (n `div` 2)
\end{code}

Divide a list into groups of given size.

\begin{code}
group :: Int -> [a] -> [[a]]
group n l = take n l:group n (drop n l)
\end{code}

The empty computation.

\begin{code}
relax :: (Monad m) => m ()
relax = return ()
\end{code}

Show or read a list, with one element per line.

\begin{code}
showsListLn :: (Show a) => [a] -> ShowS
showsListLn l =
  foldl (.) id (map (\x -> shows x . ("\n" ++)) l)

showListLn l = showsListLn l ""

readListLn :: (Read a) => String -> [a]
readListLn s = map read (lines s)
\end{code}

An approximation to the log base 2 of potentially very large
integers.

\begin{code}
two64 :: Integer
two64 = 2^64

log2 :: Integer -> Double
log2 0 = 0
log2 n | n >= two64 = 64 + log2 (shiftR n 64)
       | otherwise  = logBase 2 (fromInteger n)
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
