\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module Walk where
import Monoid
import Test.QuickCheck
import Box
\end{code}

This module is for testing the logic in Box.lhs.

A walk is a sequence of integers, starting with 0.

\begin{code}
newtype Walk = Walk [Integer] deriving (Eq, Show, Read)
\end{code}

They form a reversible monoid, as follows.

\begin{code}
instance Monoid Walk where
  unit = Walk [0]
  (Walk w1) & (Walk w2) =
     case last w1 of s -> Walk (w1 ++ map (+s) (tail w2))

instance RevMonoid Walk where
  rev (Walk w) = 
    case last w of s -> Walk (map (\x -> x - s) (reverse w))
\end{code}

The morphism from Walk to Box:

\begin{code}
walkToBox :: Walk -> Box
walkToBox (Walk w) =
  let shift = last w
      extremes = [head w, last w]
      top = maximum w
      bot = minimum w
      ntop = fromIntegral $ 2*(length (filter (== top) w)) -
	                     length (filter (== top) extremes)
      nbot = fromIntegral $ 2*(length (filter (== bot) w)) -
	                     length (filter (== bot) extremes)
  in Box shift (Reps top ntop) (Reps bot nbot)
\end{code}

\begin{code}
instance Arbitrary Walk where
  arbitrary = do { w <- arbitrary; return (Walk (0:w)) }
  coarbitrary (Walk (0:w)) = coarbitrary (tail w)
\end{code}

A convenience function for testing: specialize the generic
properties defined in Monoid.lhs for 'Walk'.

\begin{code}
testWalk :: (Testable (Walk -> a)) => (Walk -> a) -> IO ()
testWalk = test
\end{code}

Copyright 2005 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
